//////////////////////////////////////////////////////////////////////////
// This program codes an example of using the midpoint
// rectangular method to estimate the integral 
//
//          I = int[f(x),x=a..b],
//
// where f(x) is a user-defined function.  For this example
// we will assume f(x) is e^x and a=0 b=1.  We are going to
// try and "flatten" the integrand by the following scheme:
// since the midpoint rectangular method is essentially exact
// for functions that are constant or linear, we will remove the
// leading higher-than-linear terms.
// 
// For f(x)=e^x=1 + x + 0.5 x^2 + x^3/6 + ...,
// we will subtract off the third and fourth terms for the new integrand
// g(x)=e^x - 0.5 x^2 - x^3/6 and then add in the analytic terms
// int[0.5 x^2,x=a..b] + int[x^3/6,x=a..b].
//
//               thomas l. gibson  April 25, 2002
//
//    updated by thomas l. gibson  April 20, 2004
//////////////////////////////////////////////////////////////////////////
#include <cstdlib>                 // Needed for system calls
#include <iostream>                // Needed for cout and cin
#include <fstream>                 // Needed for file output
#include <iomanip>                 // Needed for setw function in file out
#include <cmath>                   // Needed for math functions
#include <cassert>                 // Needed for assert
using namespace std;

double f(double t);                // Function prototype
int main()
{
  double a,b,h,error,relerr,sum,x,exact,dn;
  int n,i,doit;
  char ask[2];
  exact = exp(1.0) - 1.0;          // Exact value of integral
  doit = 1;
  a = 0.0;                         // Lower limit of integration
  b = 1.0;                         // Upper limit of integration
  fstream prt_1;
  prt_1.open("rect_mp_flat_results.dat", ios::out);
  prt_1 << "Integration results using the flattened midpoint rectangular rule:\n";
  prt_1 << "\t for the integral int[e^x,x=0..1] = "
        << exact << "\n";
  system("clear");
  while(doit)
  {
    cout << "Input the number of panels for the midpoint rectangular rule: \n";
    cout << "(a value of 1 to 512 is suggested)\n";
    cout << "\t n = ? ";
    cin >> dn;
    n = int(dn);
    n = abs(n);
    if(n == 0)
    {
      cout << "Using 0 panels is really boring! \n";
      exit(0);
    }
    cout << "\n" << "We are using " << n <<" panels \n";
    h = (b - a)/double(n);
    sum = 0.0;
    for (i=0; i<n; i++)
    {
      x = a + (double(i) + 0.5)*h;
      sum += f(x);
    }
    sum *= h;
    sum += 5.0/24.0;      // Add back the subtracted parts (1/6 + 1/24)
    error = sum - exact;
    relerr = (error/exact)*100.0;
    cout << "For " << n << " panels, h = " << h 
         << " Int = " << sum << " error = " << error
         << " %rel. error = " << relerr << "\n";
    prt_1 << "n = " << setw(5) << n << " h = " << setw(10) << h
          << " Int = " << setw(10) << sum 
          << " error = " << setw(12) << error
          << " %rel. error = " << setw(12) << relerr << "\n";
    cout << "\n \n" << "Change the number of panels and try again? (y/n) ";
    cin >> ask;
    if(ask[0] == 'n' || ask[0] == 'N')
    {
    doit = 0;
    }
  }
  return 0;
}   
//////////////////////////////////////////////////////////////////////////
// This function calculates the integrand
// for f(x)=e^x - 0.5*x^2 - (1.0/6)*x^3
//                        thomas l. gibson  April 25, 2002
//////////////////////////////////////////////////////////////////////////
double f(double t)
{
  double y;
  y = exp(t) - 0.5*t*t - t*t*t/6.0 ;
  return y;
}
//////////////////////////////////////////////////////////////////////////
